using System;
using System.Web;
using System.Collections;

namespace SimpleModules
{
	/// <summary>
	/// Summary description for SimpleCache.
	/// </summary>
	public class SimpleCache : IHttpModule
	{
		// The stored application
		private HttpApplication mApplication;
		// Hash to store cached pages
		Hashtable mHash = new Hashtable();

		public void Init(HttpApplication app)
		{
			// Store off the application object
			mApplication = app;
			// Wire up our event handlers
			mApplication.ResolveRequestCache += new System.EventHandler(this.ResolveRequestCache);
			mApplication.UpdateRequestCache += new System.EventHandler(this.UpdateRequestCache);
		}

		public void Dispose()
		{
		}

		private void ResolveRequestCache(object sender, System.EventArgs e)
		{
			// is the url in the cache?
			if(mHash.Contains(mApplication.Request.Url))
			{
				// Write it back from the cache
				mApplication.Response.Write(mHash[mApplication.Request.Url].ToString());
				// Finish the request
				mApplication.CompleteRequest();
			}
			else
			{
				// Create a new filter
				CacheStream mStreamFilter = new CacheStream(mApplication.Response.Filter);
				// Insert it onto the page
				mApplication.Response.Filter = mStreamFilter;
				// Save a reference to the filter in the request context so we can grab it in UpdateRequestCache
				mApplication.Context.Items.Add("mStreamFilter", mStreamFilter);
			}
		}

		private void UpdateRequestCache(object sender, System.EventArgs e)
		{
			if(!mHash.Contains(mApplication.Request.Url))
			{
				// Grab the CacheStream out of the context
				CacheStream mStreamFilter = (CacheStream) mApplication.Context.Items["mStreamFilter"];
				// Remove the reference to the filter
				mApplication.Context.Items.Remove("mStreamFilter");
				// Create a buffer
				byte[] bBuffer = new byte[mStreamFilter.Length];
				// Rewind the stream
				mStreamFilter.Position = 0;
				// Get the bytes
				mStreamFilter.Read(bBuffer, 0, (int)mStreamFilter.Length);
				// Convert to a string
				System.Text.UTF8Encoding utf8 = new System.Text.UTF8Encoding();
				System.Text.StringBuilder strBuff = new System.Text.StringBuilder(utf8.GetString(bBuffer));
				// Insert the cached timestamp
				strBuff.Insert(0, "<!-- Cached: " + DateTime.Now.ToString("r") + " -->");
				// Save it away
				mHash.Add(mApplication.Request.Url, strBuff.ToString());
			}			
		}

		public class CacheStream : System.IO.Stream
		{
			private System.IO.MemoryStream moMemoryStream = new System.IO.MemoryStream();
			private System.IO.Stream moStream;

			public CacheStream(System.IO.Stream stream)
			{
				moStream = stream;
			}

			public override bool CanRead
			{
				get
				{
					return true;
				}
			}

			public override bool CanWrite
			{
				get
				{
					return true;
				}
			}

			public override bool CanSeek
			{
				get
				{
					return true;
				}
			}

			public override long Length
			{
				get
				{
					return moMemoryStream.Length;
				}
			}

			public override long Position
			{
				get
				{
					return moMemoryStream.Position;
				}
				set 
				{
					moMemoryStream.Position = value;
				}
			}

			public override int Read(byte[] buffer, int offset, int count)
			{
				return moMemoryStream.Read(buffer, offset, count);
			}

			public override long Seek(long offset, System.IO.SeekOrigin direction)
			{
				return moMemoryStream.Seek(offset, direction);
			}

			public override void SetLength(long length)
			{
				moMemoryStream.SetLength(length);
			}

			public override void Close() 
			{
				moStream.Close();
			}

			public override void Flush()
			{
				moStream.Flush();
			}

			public override void Write(byte[] buffer, int offset, int count)
			{
				moStream.Write(buffer, offset, count);
				moMemoryStream.Write(buffer, offset, count);
			}
		}
	}
}
